# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

sys.path.insert(0, os.path.abspath("../"))

# -- Project information -----------------------------------------------------
project = "PyABSA"
copyright = "2020, Heng Yang"
author = "Heng Yang"

# The full version, including alpha/beta/rc tags
release = "2.0.20"

# Set master doc to `index.rst`.
master_doc = "index"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.viewcode",
    "sphinx.ext.autodoc",
    "sphinx.ext.inheritance_diagram",
    "sphinx.ext.autosummary",
    "sphinx.ext.napoleon",
    "sphinx_rtd_theme",
    "sphinx_copybutton",
    # Enable .ipynb doc files
    "nbsphinx",
    # Enable .md doc files
    "recommonmark",
    "sphinx_markdown_tables",
    "IPython.sphinxext.ipython_console_highlighting",
]
autosummary_generate = True

# Add any paths that contain templates here, relative to this directory.
templates_path = []

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "**.ipynb_checkpoints"]

# Mock expensive textattack imports. Docs imports are in `docs/requirements.txt`.
autodoc_mock_imports = []

# Output file base name for HTML help builder.
htmlhelp_basename = "PyABASA_documentation"


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"
html_theme_options = {
    "collapse_navigation": False,
}

html_static_path = ["_static"]
html_css_files = [
    "css/custom.css",
]

html_sidebars = {
    "**": ["globaltoc.html", "relations.html", "sourcelink.html", "searchbox.html"]
}

# Path to favicon.
html_favicon = "favicon.png"

# Don't show module names in front of class names.
add_module_names = True

# Sort members by group
autodoc_member_order = "groupwise"

# -- Options for Sphinx Copy Button-------------------------------------------------

# Exclude the prompt symbol ">>>" when copying text
copybutton_prompt_text = ">>> "

# Specify Line Continuation Character so that all the entire Line is copied
copybutton_line_continuation_character = "\\"
